@@ -5,6 +5,7 @@ use warnings;
 use base 'Exporter';
 use vars '@EXPORT_OK';
 
+use Dancer::Deprecation;
 use Dancer::Template;
 use Dancer::ModuleLoader;
 use Dancer::FileUtils 'path';
@@ -14,16 +15,11 @@ use Encode;
 
 @EXPORT_OK = qw(setting mime_types);
 
+my $SETTINGS = {};
+
 # mergeable settings
 my %MERGEABLE = map { ($_ => 1) } qw( plugins handlers );
 
-# singleton for storing settings
-my $SETTINGS = {
-
-    # user defined mime types
-    mime_types => {},
-};
-
 sub settings {$SETTINGS}
 
 my $setters = {
@@ -61,7 +57,7 @@ my $setters = {
                 'get', '/:page',
                 sub {
                     my $params = Dancer::SharedData->request->params;
-                    Dancer::Helpers::template($params->{'page'});
+                    Dancer::template($params->{'page'});
                 }
             );
         }
@@ -89,6 +85,16 @@ my $normalizers = {
     },
 };
 
+sub mime_types {
+    Dancer::Deprecation->deprecated(
+        reason => "use 'mime_type' from Dancer.pm",
+    );
+    my $mime = Dancer::MIME->instance();
+    if    (scalar(@_)==2) { $mime->add_mime_type(@_) }
+    elsif (scalar(@_)==1) { $mime->mime_type_for(@_) }
+    else                  { $mime->aliases           }
+}
+
 sub normalize_setting {
     my ($class, $setting, $value) = @_;
 
@@ -135,16 +141,6 @@ sub _get_setting {
     return $SETTINGS->{$setting};
 }
 
-sub mime_types {
-    my ($ext, $content_type) = @_;
-    $SETTINGS->{mime_types} ||= {};
-    return $SETTINGS->{mime_types} if @_ == 0;
-
-    return (@_ == 2)
-      ? $SETTINGS->{mime_types}{$ext} = $content_type
-      : $SETTINGS->{mime_types}{$ext};
-}
-
 sub conffile { path(setting('confdir') || setting('appdir'), 'config.yml') }
 
 sub environment_file {
@@ -213,8 +209,9 @@ sub load_default_settings {
     $SETTINGS->{warnings}     ||= $ENV{DANCER_WARNINGS}     || 0;
     $SETTINGS->{auto_reload}  ||= $ENV{DANCER_AUTO_RELOAD}  || 0;
     $SETTINGS->{traces}       ||= $ENV{DANCER_TRACES}       || 0;
-    $SETTINGS->{environment} 
-      ||= $ENV{DANCER_ENVIRONMENT}
+    $SETTINGS->{logger}       ||= $ENV{DANCER_LOGGER}       || 'file';
+    $SETTINGS->{environment} ||=
+         $ENV{DANCER_ENVIRONMENT}
       || $ENV{PLACK_ENV}
       || 'development';
 
