@@ -2,11 +2,14 @@ package Dancer::Cookie;
 use strict;
 use warnings;
 
+use URI::Escape;
+
 use base 'Dancer::Object';
-__PACKAGE__->attributes('name', 'value', 'expires', 'domain', 'path');
+__PACKAGE__->attributes('name', 'expires', 'domain', 'path', "secure");
 
 sub init {
-    my $self = shift;
+    my ($self, %args) = @_;
+    $self->value($args{value});
     if ($self->expires) {
         $self->expires(_epoch_to_gmtstring($self->expires))
           if $self->expires =~ /^\d+$/;
@@ -17,11 +20,29 @@ sub init {
 sub to_header {
     my $self   = shift;
     my $header = '';
-    $header .= $self->name . '=' . $self->value . '; ';
-    $header .= "path=" . $self->path . "; " if $self->path;
-    $header .= "expires=" . $self->expires . "; " if $self->expires;
-    $header .= "domain=" . $self->domain . "; " if $self->domain;
-    $header .= 'HttpOnly';
+
+    my $value = join('&', map {uri_escape($_)} $self->value);
+
+    my @headers = $self->name . '=' . $value;
+    push @headers, "path=" . $self->path        if $self->path;
+    push @headers, "expires=" . $self->expires  if $self->expires;
+    push @headers, "domain=" . $self->domain    if $self->domain;
+    push @headers, "Secure"                     if $self->secure;
+    push @headers, 'HttpOnly';
+
+    return join '; ', @headers;
+}
+
+sub value {
+    my ( $self, $value ) = @_;
+    if ( defined $value ) {
+        my @values =
+            ref $value eq 'ARRAY' ? @$value
+          : ref $value eq 'HASH'  ? %$value
+          :                         ($value);
+        $self->{'value'} = [@values];
+    }
+    return wantarray ? @{ $self->{'value'} } : $self->{'value'}->[0];
 }
 
 sub _epoch_to_gmtstring {
@@ -63,11 +84,11 @@ Dancer::Cookie provides a HTTP cookie object to work with cookies.
 
 =head1 ATTRIBUTES
 
-=head3 name
+=head2 name
 
 The cookie's name.
 
-=head3 value
+=head2 value
 
 The cookie's value.
 
@@ -83,6 +104,11 @@ The cookie's domain.
 
 The cookie's path.
 
+=head2 secure
+
+If true, it instructs the client to only serve the cookie over secure
+connections such as https.
+
 =head1 METHODS/SUBROUTINES
 
 =head2 new
