@@ -2,41 +2,46 @@ package Dancer::Error;
 
 use strict;
 use warnings;
+use Carp;
+
+use base 'Dancer::Object';
 
 use Dancer::Response;
 use Dancer::Renderer;
 use Dancer::Config 'setting';
 use Dancer::Logger;
 use Dancer::Session;
+use Dancer::FileUtils qw(open_file);
 
-sub new {
-    my ($class, %params) = @_;
-    my $self = \%params;
-    bless $self, $class;
+sub init {
+    my ($self) = @_;
 
-    $self->{title} ||= "Error " . $self->code;
-    $self->{type}  ||= "runtime error";
+    $self->attributes_defaults(
+        title => 'Error ' . $self->code,
+        type  => 'runtime error',
+    );
 
-    if (!$self->has_serializer) {
-        my $html_output = "<h2>" . $self->{type} . "</h2>";
-        $html_output .= $self->backtrace;
-        $html_output .= $self->environment;
+    $self->has_serializer
+      and return;
 
-        $self->{message} = $html_output;
-    }
-    return $self;
+    my $html_output = "<h2>" . $self->{type} . "</h2>";
+    $html_output .= $self->backtrace;
+    $html_output .= $self->environment;
+
+    $self->{message} = $html_output;
 }
 
 sub has_serializer { setting('serializer') }
-sub code    { $_[0]->{code} }
-sub title   { $_[0]->{title} }
-sub message { $_[0]->{message} }
+sub code           { $_[0]->{code} }
+sub title          { $_[0]->{title} }
+sub message        { $_[0]->{message} }
 
 sub backtrace {
     my ($self) = @_;
 
     $self->{message} ||= "";
-    my $message = qq|<pre class="error">| . _html_encode($self->{message}) . "</pre>";
+    my $message =
+      qq|<pre class="error">| . _html_encode($self->{message}) . "</pre>";
 
     # the default perl warning/error pattern
     my ($file, $line) = ($message =~ /at (\S+) line (\d+)/);
@@ -49,8 +54,7 @@ sub backtrace {
     return $message unless ($file and $line);
 
     # file and line are located, let's read the source Luke!
-    my $fh;
-    open $fh, '<', $file or return $message;
+    my $fh = open_file('<', $file) or return $message;
     my @lines = <$fh>;
     close $fh;
 
@@ -67,7 +71,7 @@ sub backtrace {
 
     for (my $l = $start; $l <= $stop; $l++) {
         chomp $lines[$l];
-        
+
         if ($l == $line) {
             $backtrace
               .= qq|<span class="nu">|
@@ -100,20 +104,21 @@ sub tabulate {
 sub dumper {
     my $obj = shift;
     return "Unavailable without Data::Dumper"
-        unless Dancer::ModuleLoader->load('Data::Dumper');
+      unless Dancer::ModuleLoader->load('Data::Dumper');
 
 
     # Take a copy of the data, so we can mask sensitive-looking stuff:
-    my %data = %$obj;
-    my $censored = _censor(\%data); 
-   
+    my %data     = %$obj;
+    my $censored = _censor(\%data);
+
     #use Data::Dumper;
     my $dd = Data::Dumper->new([\%data]);
     $dd->Terse(1)->Quotekeys(0)->Indent(1);
     my $content = $dd->Dump();
     $content =~ s{(\s*)(\S+)(\s*)=>}{$1<span class="key">$2</span>$3 =&gt;}g;
     if ($censored) {
-        $content .= "\n\nNote: Values of $censored sensitive-looking keys hidden\n";
+        $content
+          .= "\n\nNote: Values of $censored sensitive-looking keys hidden\n";
     }
     return $content;
 }
@@ -123,7 +128,7 @@ sub dumper {
 sub _censor {
     my $hash = shift;
     if (!$hash || ref $hash ne 'HASH') {
-        warn "_censor given incorrect input: $hash";
+        carp "_censor given incorrect input: $hash";
         return;
     }
 
@@ -131,7 +136,8 @@ sub _censor {
     for my $key (keys %$hash) {
         if (ref $hash->{$key} eq 'HASH') {
             $censored += _censor($hash->{$key});
-        } elsif ($key =~ /(pass|card?num|pan|secret)/i) {
+        }
+        elsif ($key =~ /(pass|card?num|pan|secret)/i) {
             $hash->{$key} = "Hidden (looks potentially sensitive)";
             $censored++;
         }
@@ -147,7 +153,7 @@ sub _html_encode {
     $value =~ s/&/&amp;/g;
     $value =~ s/</&lt;/g;
     $value =~ s/>/&gt;/g;
-    $value =~ s/'/&#39;/g;  
+    $value =~ s/'/&#39;/g;
     $value =~ s/"/&quot;/g;
 
     return $value;
@@ -167,11 +173,23 @@ sub _render_serialized {
     my $message =
       !ref $self->message ? {error => $self->message} : $self->message;
 
-    Dancer::Response->new(
-        status  => $self->code,
-        content => Dancer::Serializer->engine->serialize($message),
-        headers => ['Content-Type' => Dancer::Serializer->engine->content_type]
-    );
+    if (setting('show_errors')) {
+        Dancer::Response->new(
+            status  => $self->code,
+            content => Dancer::Serializer->engine->serialize($message),
+            headers => ['Content-Type' => Dancer::Serializer->engine->content_type]
+            );
+    }
+    
+    # if show_errors is disabled, we don't expose the real error message to the
+    # outside world
+    else {
+        Dancer::Response->new(
+            status => $self->code,
+            content => "An internal error occured",
+        );
+    }
+
 }
 
 sub _render_html {
@@ -191,7 +209,7 @@ sub environment {
     my ($self) = @_;
 
     my $request = Dancer::SharedData->request;
-    my $r_env = {};
+    my $r_env   = {};
     $r_env = $request->env if defined $request;
 
     my $env =
@@ -207,11 +225,12 @@ sub environment {
       . $self->get_caller
       . "</pre>";
     my $session = "";
+
     if (setting('session')) {
-        $session = 
+        $session =
             qq[<div class="title">Session</div><pre class="content">]
-            . dumper(  Dancer::Session->get  )
-            . "</pre>";
+          . dumper(Dancer::Session->get)
+          . "</pre>";
     }
     return "$source $settings $session $env";
 }
@@ -248,7 +267,7 @@ Dancer::Error - class for representing fatal errors
         message => "No such file: `$path'"
     );
 
-    Dancer::Response::set($error->render);
+    Dancer::Response->set($error->render);
 
 =head1 DESCRIPTION
 
@@ -341,6 +360,7 @@ C<dumper> calls this method to censor things like passwords and such.
 
 Internal method to encode entities that are illegal in (X)HTML. We output as
 UTF-8, so no need to encode all non-ASCII characters or use a module.
+FIXME : this is not true anymore, output can be any charset. Need fixing.
 
 =head1 AUTHOR
 
