@@ -5,6 +5,8 @@ use warnings;
 
 use Dancer::Config 'setting';
 use Getopt::Long;
+use FindBin;
+use File::Spec;
 
 my $options = {
     port        => setting('port'),
@@ -35,8 +37,9 @@ sub process_args {
 sub usage_and_exit { print_usage() && exit(0) }
 
 sub print_usage {
+    my $app = File::Spec->catfile( $FindBin::RealBin, $FindBin::RealScript );
     print <<EOF
-\$ ./yourdancerapp.pl [options]
+\$ $app [options]
 
  Options:
    --daemon             Run in background (false)
@@ -76,7 +79,7 @@ Which environment to use. By default this value is set to development.
 
 --restart
 
-Set the value of the B<auto_reload> setting. Usefull when you want to switch
+Set the value of the B<auto_reload> setting. Useful when you want to switch
 this setting for a test without changing the value in your configurations
 file.
 
