@@ -15,20 +15,14 @@ use Dancer::Config 'setting';
 use Dancer::FileUtils qw(path dirname read_file_content open_file);
 use Dancer::SharedData;
 use Dancer::Logger;
-use MIME::Types;
+use Dancer::MIME;
 
-BEGIN {
-    MIME::Types->new(only_complete => 1);
-}
-
-sub render_file {
-    return get_file_response();
-}
+sub render_file { get_file_response() }
 
 sub render_action {
     my $resp = get_action_response();
     return (defined $resp)
-      ? response_with_headers($resp)
+      ? response_with_headers()
       : undef;
 }
 
@@ -53,18 +47,10 @@ sub render_error {
 
 # Takes a response object and add default headers
 sub response_with_headers {
-    my $response = shift;
+    my $response = Dancer::SharedData->response();
 
-    $response->{headers} ||= HTTP::Headers->new;
-    $response->header('X-Powered-By' => "Perl Dancer ${Dancer::VERSION}");
-
-    # add cookies
-    foreach my $c (keys %{Dancer::Cookies->cookies}) {
-        my $cookie = Dancer::Cookies->cookies->{$c};
-        if (Dancer::Cookies->has_changed($cookie)) {
-            $response->header('Set-Cookie' => $cookie->to_header);
-        }
-    }
+   $response->{headers} ||= HTTP::Headers->new;
+   $response->header('X-Powered-By' => "Perl Dancer ${Dancer::VERSION}");
 
     return $response;
 }
@@ -87,7 +73,7 @@ sub html_page {
 }
 
 sub get_action_response {
-    my $response;
+    my $depth = shift || 1;
 
     # save the request before the filters are ran
     my $request = Dancer::SharedData->request;
@@ -103,41 +89,39 @@ sub get_action_response {
     $_->() for @{$app->registry->hooks->{before}};
 
     # recurse if something has changed
-    my $limit              = 0;
     my $MAX_RECURSIVE_LOOP = 10;
     if (   ($path ne Dancer::SharedData->request->path_info)
         || ($method ne Dancer::SharedData->request->method))
     {
-        $limit++;
-        if ($limit > $MAX_RECURSIVE_LOOP) {
+        if ($depth > $MAX_RECURSIVE_LOOP) {
             croak "infinite loop detected, "
               . "check your route/filters for "
               . $method . ' '
               . $path;
         }
-        return get_action_response();
+        return get_action_response($depth + 1);
     }
 
     # redirect immediately - skip route execution
-    if (my $status = Dancer::Response->status) {
+    my $response = Dancer::SharedData->response();
+    if (defined $response && (my $status = $response->status)) {
         if ($status == 302 || $status == 301) {
-            return serialize_response_if_needed(Dancer::Response->current);
+            serialize_response_if_needed();
+            return $response;
         }
     }
 
     # execute the action
     if ($handler) {
-
         # a response may exist, produced by a before filter
-        return serialize_response_if_needed(Dancer::Response->current)
-          if Dancer::Response->exists;
-
+        return serialize_response_if_needed() if defined $response && $response->exists;
         # else, get the route handler's response
         Dancer::App->current($handler->app);
-        $response = $handler->run($request);
-        $response = serialize_response_if_needed($response);
-        $_->($response) for (@{$app->registry->hooks->{after}});
-        return $response;
+        $handler->run($request);
+        serialize_response_if_needed();
+        my $resp = Dancer::SharedData->response();
+        $_->($resp) for (@{$app->registry->hooks->{after}});
+        return $resp;
     }
     else {
         return;    # 404
@@ -145,13 +129,12 @@ sub get_action_response {
 }
 
 sub serialize_response_if_needed {
-    my ($response) = @_;
+    my $response = Dancer::SharedData->response();
     $response = Dancer::Serializer->process_response($response)
       if Dancer::App->current->setting('serializer') && $response->content();
     return $response;
 }
 
-
 sub get_file_response {
     my $request     = Dancer::SharedData->request;
     my $path_info   = $request->path_info;
@@ -164,15 +147,14 @@ sub get_file_response_for_path {
     my ($class, $static_file, $status) = @_;
     $status ||= 200;
 
-    if (-f $static_file) {
-        my $fh = open_file('<', $static_file);
+    if ( -f $static_file ) {
+        my $fh = open_file( '<', $static_file );
         binmode $fh;
-
-        return Dancer::Response->new(
-            status  => $status,
-            headers => ['Content-Type' => get_mime_type($static_file)],
-            content => $fh
-        );
+        my $response = Dancer::SharedData->response() || Dancer::Response->new();
+        $response->status($status);
+        $response->header('Content-Type' => get_mime_type($static_file));
+        $response->content($fh);
+        return $response;
     }
     return;
 }
@@ -181,18 +163,11 @@ sub get_file_response_for_path {
 
 sub get_mime_type {
     my ($filename) = @_;
-    my @tokens = reverse(split(/\./, $filename));
-    my $ext = $tokens[0];
+    my ($ext) = $filename =~ /\.([^.]+)$/;
+    return 'application/data' unless $ext;
 
-    # first check user configured mime types
-    my $mime = Dancer::Config::mime_types($ext);
-    return $mime if defined $mime;
-
-    # user has not specified a mime type, so ask MIME::Types
-    $mime = MIME::Types->new(only_complete => 1)->mimeTypeOf($ext);
-
-    # default to text/plain
-    return defined($mime) ? $mime : 'text/plain';
+    my $mime = Dancer::MIME->instance();
+    return $mime->mime_type_for($ext);
 }
 
 # set of builtin templates needed by Dancer when rendering HTML pages
