@@ -77,8 +77,10 @@ sub match {
           . "/");
 
     my @values = $path =~ $self->{_compiled_regexp};
-    Dancer::Logger::core("  --> got @values") if @values;
-
+    Dancer::Logger::core("  --> got ".
+        map { defined $_ ? $_ : 'undef' } @values) 
+        if @values;
+    
     # if some named captures found, return captures
     # no warnings is for perl < 5.10
     if (my %captures =
@@ -151,16 +153,11 @@ sub run {
     my ($self, $request) = @_;
 
     my $content  = $self->execute();
-    my $response = Dancer::Response->current;
+    my $response = Dancer::SharedData->response;
 
-    if ( $response->is_forwarded ) {
-        my $new_req = Dancer::Request->new_for_request(
-            $request->method,
-            $response->{forward},
-            $request->params,
-            $request->body,
-            $request->headers,
-        );
+    if ( $response && $response->is_forwarded ) {
+        my $new_req = 
+            Dancer::Request->forward($request, $response->{forward});
 
         my $marshalled = Dancer::Handler->handle_request($new_req);
 
@@ -171,7 +168,8 @@ sub run {
         );
     }
 
-    if ($response->has_passed) {
+    if ($response && $response->has_passed) {
+        $response->pass(0);
         if ($self->next) {
             my $next_route = $self->find_next_matching_route($request);
             return $next_route->run($request);
@@ -185,14 +183,15 @@ sub run {
     # prevent warnings
     $content = (defined $content) ? $content : '';
 
-    # drop content if HEAD request
-    $content = '' if $request->is_head;
+    my $ct =
+      ( defined $response && defined $response->content_type )
+      ? $response->content_type()
+      : setting('content_type');
+
+    my $st = defined $response ? $response->status : 200;
 
-    # init response headers
-    my $ct = $response->content_type() || setting('content_type');
-    my $st = $response->status()       || 200;
     my $headers = [];
-    push @$headers, @{$response->headers_to_array};
+    push @$headers, @{ $response->headers_to_array } if defined $response;
 
     # content type may have already be set earlier
     # (eg: with send_error)
@@ -225,7 +224,7 @@ sub execute {
         my $content = $self->code->();
         if ($warning) {
             return Dancer::Error->new(
-                status  => 500,
+                code    => 500,
                 message => "Warning caught during route execution: $warning",
             )->render;
         }
@@ -252,8 +251,9 @@ sub _init_prefix {
         # - /prefix/
         # - /prefix
         # this is done by creating a regex for this case
-        my $pattern = $self->pattern;
-        my $regex   = qr/$prefix(?:$pattern)?$/;
+        my $qpattern = quotemeta( $self->pattern );
+        my $qprefix  = quotemeta( $self->prefix );
+        my $regex    = qr/^$qprefix(?:$qpattern)?$/;
         $self->{regexp}  = $regex;
         $self->{pattern} = $regex;
     }
@@ -301,9 +301,9 @@ sub _build_regexp_from_string {
 
     # look for route with params (/hello/:foo)
     if ($pattern =~ /:/) {
-        @params = $pattern =~ /:([^\/\.]+)/g;
+        @params = $pattern =~ /:([^\/\.\?]+)/g;
         if (@params) {
-            $pattern =~ s/(:[^\/\.]+)/\(\[\^\/\]\+\)/g;
+            $pattern =~ s/(:[^\/\.\?]+)/\(\[\^\/\]\+\)/g;
             $capture = 1;
         }
     }
