@@ -14,13 +14,17 @@ my $_engine;
 sub init {
     my ($self) = @_;
 
-    croak "Template is needed by Dancer::Template::TemplateToolkit"
-      unless Dancer::ModuleLoader->load('Template');
+    my $class = $self->config->{subclass} || "Template";
+    croak "$class is needed by Dancer::Template::TemplateToolkit"
+      if !$class->can("process") and !Dancer::ModuleLoader->load($class);
+
+    my $charset = setting('charset') || '';
+    my @encoding = length($charset) ? ( ENCODING => $charset ) : ();
 
     my $tt_config = {
         ANYCASE  => 1,
         ABSOLUTE => 1,
-        ENCODING => 'utf8',
+        @encoding,
         %{$self->config},
     };
 
@@ -37,16 +41,20 @@ sub init {
 
     $tt_config->{INCLUDE_PATH} = setting('views');
 
-    $_engine = Template->new(%$tt_config);
+    $_engine = $class->new(%$tt_config);
 }
 
 sub render {
     my ($self, $template, $tokens) = @_;
-    croak "'$template' is not a regular file"
-      if !ref($template) && (!-f $template);
+
+    if ( ! ref $template ) {
+        -f $template or croak "'$template' doesn't exist or not a regular file";
+    }
 
     my $content = "";
-    $_engine->process($template, $tokens, \$content, binmode => ':utf8') or croak $_engine->error;
+    my $charset = setting('charset') || '';
+    my @options = length($charset) ? ( binmode => ":encoding($charset)" ) : ();
+    $_engine->process($template, $tokens, \$content, @options) or croak $_engine->error;
     return $content;
 }
 
@@ -84,6 +92,13 @@ within your config file - for example:
             start_tag: '[%'
             stop_tag: '%]'
 
+By default, L<Template> is used, but you can configure Dancer to use a
+subclass with the C<subclass> option.
+
+    engines:
+        template_toolkit:
+            subclass: My::Template
+
 
 =head1 SEE ALSO
 
