@@ -10,6 +10,7 @@ use Carp;
 use HTTP::Headers;
 use Dancer ':syntax';
 use Dancer::App;
+use Dancer::Deprecation;
 use Dancer::Request;
 use Dancer::SharedData;
 use Dancer::Renderer;
@@ -39,11 +40,13 @@ use vars '@EXPORT';
 
   dancer_response
   get_response
+
+  read_logs
 );
 
 sub import {
     my ($class, %options) = @_;
-    $options{appdir} ||= '..';
+    $options{appdir} ||= '.';
 
     # mimic PSGI env
     $ENV{SERVERNAME}        = 'localhost';
@@ -54,10 +57,15 @@ sub import {
     my ($package, $script) = caller;
     $class->export_to_level(1, $class, @EXPORT);
 
+    Dancer::_init_script_dir($options{appdir});
+    Dancer::Config->load;
+
     # set a default session engine for tests
     setting 'session' => 'simple';
-    Dancer::_init($options{appdir});
-    Dancer::Config->load;
+
+    # capture logs for testing
+    setting 'logger'  => 'capture';
+    setting 'log'     => 'debug';
 }
 
 # Route Registry
@@ -105,7 +113,7 @@ sub response_status_is {
     $test_name ||= "response status is $status for @$req";
 
     my $response = dancer_response(@$req);
-    is $response->{status}, $status, $test_name;
+    is $response->status, $status, $test_name;
 }
 
 sub response_status_isnt {
@@ -180,9 +188,22 @@ sub dancer_response {
 
     if ($method =~ /^(?:PUT|POST)$/ && $args->{body}) {
         my $body = $args->{body};
-        my $l    = length $body;
+
+        # coerce hashref into an url-encoded string
+        if (ref($body) && (ref($body) eq 'HASH')) {
+            my @tokens;
+            while (my ($name, $value) = each %{$body}) {
+                $name  = _url_encode($name);
+                $value = _url_encode($value);
+                push @tokens, "${name}=${value}";
+            }
+            $body = join('&', @tokens);
+        }
+
+        my $l = length $body;
         open my $in, '<', \$body;
         $ENV{'CONTENT_LENGTH'} = $l;
+        $ENV{'CONTENT_TYPE'}   = 'application/x-www-form-urlencoded';
         $ENV{'psgi.input'}     = $in;
     }
 
@@ -202,16 +223,31 @@ sub dancer_response {
     );
 
     Dancer::SharedData->request($request);
-    return Dancer::Renderer::get_action_response();
+
+    my $get_action = Dancer::Renderer::get_action_response();
+    my $response = Dancer::SharedData->response();
+    $response->content('') if $method eq 'HEAD';
+    Dancer::SharedData->reset_response();
+    return $response if $get_action;
+    (defined $response && $response->exists) ? return $response : return undef;
 }
 
 sub get_response {
-    carp "get_response() is DEPRECATED. Use dancer_response() instead.";
-    return dancer_response(@{$_[0]});
+    Dancer::Deprecation->deprecated(
+        fatal   => 1,
+        feature => 'get_response',
+        reason  => 'Use dancer_response() instead.',
+    );
 }
 
 # private
 
+sub _url_encode {
+    my $string = shift;
+    $string =~ s/([\W])/"%" . uc(sprintf("%2.2x",ord($1)))/eg;
+    return $string;
+}
+
 sub _get_file_response {
     my ($req) = @_;
     my ($method, $path, $params) = @$req;
@@ -227,6 +263,11 @@ sub _get_handler_response {
     return Dancer::Handler->handle_request($request);
 }
 
+sub read_logs {
+    return Dancer::Logger::Capture->trap->read;
+}
+
+
 1;
 __END__
 
@@ -243,7 +284,7 @@ Dancer::Test - Test helpers to test a Dancer application
     use Test::More tests => 2;
 
     use MyWebApp;
-    use Dancer::Test appdir => '..';
+    use Dancer::Test;
 
     response_status_is [GET => '/'], 200, "GET / is found";
     response_content_like [GET => '/'], qr/hello, world/, "content looks good for /";
@@ -253,19 +294,7 @@ Dancer::Test - Test helpers to test a Dancer application
 
 This module provides test helpers for testing Dancer apps.
 
-=head1 CONFIGURATON
-
-When importing Dancer::Test, the appdir is set by defaut to '..', assuming that
-your test script is directly in your t/ directory. If you put your test script
-deeper in the 't/' hierarchy (like in 't/routes/01_some_test.t') you'll have to
-tell Dancer::Test that the appdir is one step upper.
-
-To do so, you can tell where the appdir is thanks to an import option:
-
-    use MyWebApp;
-    use Dancer::Test appdir => '../..';
-
-Be careful, the order in the example above is very important.
+Be careful, the module loading order in the example above is very important.
 Make sure to use C<Dancer::Test> B<after> importing the application package
 otherwise your appdir will be automatically set to C<lib> and your test script
 won't be able to find views, conffiles and other application content.
@@ -328,7 +357,7 @@ Asserts that the response content is equal to the C<$expected> string.
 
 Asserts that the response content is not equal to the C<$not_expected> string.
 
-    response_content_is [GET => '/'], "Hello, World", 
+    response_content_isnt [GET => '/'], "Hello, World", 
         "got expected response content for GET /";
 
 =head2 response_content_is_deeply([$method, $path], $expected_struct, $test_name)
@@ -393,6 +422,32 @@ Schrodinger's cat to die.
 
 This method is B<DEPRECATED>.  Use dancer_response() instead.
 
+
+=head2 read_logs
+
+    my $logs = read_logs;
+
+Returns an array ref of all log messages issued by the app since the
+last call to C<read_logs>.
+
+For example:
+
+    warning "Danger!  Warning!";
+    debug   "I like pie.";
+
+    is_deeply read_logs, [
+        { level => "warning", message => "Danger!  Warning!" },
+        { level => "debug",   message => "I like pie.", }
+    ];
+
+    error "Put out the light.";
+
+    is_deeply read_logs, [
+        { level => "error", message => "Put out the light." },
+    ];
+
+See L<Dancer::Logger::Capture> for more details.
+
 =head1 LICENSE
 
 This module is free software and is distributed under the same terms as Perl
